#!/usr/bin/env python3
"""
An NRPE check for juju controllers.

This NRPE check queries the juju controller endpoint and returns the following:
  * UNKNOWN for unsupported juju versions
  * WARNING during controller startup/shutdown
  * OK when healthy
  * CRITICAL in any other case
"""

import sys
from enum import Enum

import requests
from requests.exceptions import RequestException

import semver

import yaml

# Code duplication, I know, but importing from lib_juju_controller.py is not
# trivial, as that file's path depends on the application name. This could be
# solved by making this whole script a jinja template rendered at charm
# installation, but this in turn would make unit testing more complex.
CACERTPATH = "/run/juju-controller-ca.crt"
# This file will only contain non-secret values extracted from the controller
# agent.conf.
# At the moment these are only the juju version and the API port.
PUBAGENTCONFPATH = "/run/juju-controller-public-agent.conf"


# see:
# - https://github.com/juju/juju/pull/11096
# - https://bugs.launchpad.net/juju/+bug/1797848
MINVER = "2.7.2"
# Known response bodies that may be returned by the health endpoint
OKSTATUSES = ["running"]
WARNSTATUSES = ["starting", "stopping"]


class States(Enum):
    """Class representing valid Nagios states and their return codes."""

    OK = 0
    WARNING = 1
    CRITICAL = 2
    UNKNOWN = 3


def _supported(version):
    return semver.compare(version, MINVER) >= 0


def _get_agent_conf(path=PUBAGENTCONFPATH):
    with open(path, "r") as agentconffile:
        return yaml.safe_load(agentconffile)


def check_endpoint(agentconf=PUBAGENTCONFPATH):
    """Run a GET against the controller endpoint and return its status."""
    try:
        agentconf = _get_agent_conf(agentconf)
    except FileNotFoundError:
        msg = "{} not found. Cannot query the endpoint".format(PUBAGENTCONFPATH)
        return States.UNKNOWN, msg

    endpoint = "https://localhost:{}/health".format(agentconf.get("apiport"))

    controller_version = agentconf.get("upgradedToVersion")
    if not _supported(controller_version):
        msg = "juju version {} does not offer a monitoring endpoint".format(
            controller_version
        )
        return States.UNKNOWN, msg

    try:
        r = requests.get(endpoint, verify=CACERTPATH, timeout=15)
    except (RequestException, OSError) as error:
        msg = "cannot connect to the health endpoint. Error: {}".format(error)
        return States.CRITICAL, msg

    if not r.ok:
        msg = "the health endpoint returned status code {}".format(r.status_code)
        return States.UNKNOWN, msg

    controller_status = r.text.strip()
    if controller_status in WARNSTATUSES:
        msg = 'juju is in state "{}"'.format(controller_status)
        return States.WARNING, msg
    elif controller_status in OKSTATUSES:
        msg = 'juju is in state "{}"'.format(controller_status)
        return States.OK, msg
    else:
        msg = 'juju is in state "{}"'.format(controller_status)
        return States.CRITICAL, msg


def main():
    """Print the status of the controller and return the proper RC."""
    state, msg = check_endpoint()
    formatted_output = "{}: {}".format(state.name, msg)
    print(formatted_output)
    sys.exit(state.value)


if __name__ == "__main__":
    main()
