"""A small helper lib for the juju controller charm."""

import glob
import os
import shutil

from charmhelpers.core import hookenv

import yaml

AGENTDIRGLOB = "/var/lib/juju/agents/machine-*"
CACERTPATH = "/run/juju-controller-ca.crt"
PUBAGENTCONFPATH = "/run/juju-controller-public-agent.conf"


class JujuControllerHelper:
    """A small helper lib for the juju controller charm."""

    def __init__(self):
        """Initialize the helper by reading the charm config."""
        self.charm_config = hookenv.config()

    def _machine_id(self):
        machine = os.path.basename(glob.glob(AGENTDIRGLOB)[0])
        return machine.split("-").pop()

    def dump_ca(self, path=CACERTPATH):
        """Extract CA cert from agent.conf and place it in a nagios-readable file."""
        with open(path, "w") as cafile:
            cafile.write(self.cacert)
        try:
            shutil.chown(path, user="nagios")
        except LookupError as error:
            msg = "cannot chown {} to user nagios. {}".format(path, error)
            hookenv.log(msg, level=hookenv.WARNING)

    def dump_public_agent_conf(self, path=PUBAGENTCONFPATH):
        """Provide version and api port to nagios.

        Get version and API port from agent.conf
        and place them in a nagios-readable file.
        """
        with open(path, "w") as pubagentconffile:
            public_data = {
                "upgradedToVersion": self.version,
                "apiport": self.apiport,
            }
            yaml.dump(public_data, pubagentconffile)
        try:
            shutil.chown(path, user="nagios")
        except LookupError as error:
            msg = "cannot chown {} to user nagios. {}".format(path, error)
            hookenv.log(msg, level=hookenv.WARNING)

    @property
    def services(self):
        """Return the list of juju services on this machine."""
        return ["juju-db", "jujud-machine-{}".format(self._machine_id())]

    @property
    def agent_conf_path(self):
        """Return the path of the controller agent.conf file."""
        return os.path.join(glob.glob(AGENTDIRGLOB)[0], "agent.conf")

    @property
    def agent_conf(self):
        """Return the content of the controller agent.conf file as a map."""
        with open(self.agent_conf_path, "r") as agentconffile:
            return yaml.safe_load(agentconffile)

    @property
    def version(self):
        """Return the version of the juju controller."""
        return self.agent_conf.get("upgradedToVersion")

    @property
    def apiport(self):
        """Return the tcp port the API is listening at."""
        return self.agent_conf.get("apiport")

    @property
    def cacert(self):
        """Return the CA certificate needed to verify the juju controller cert."""
        return self.agent_conf.get("cacert")
