#!/usr/bin/env python3
"""Functional tests module."""
import json
import os
import unittest
from pathlib import Path

from zaza import model, sync_wrapper
from zaza.charm_lifecycle import utils as lifecycle_utils


CHARM_BUILD_DIR = os.environ.get("CHARM_BUILD_DIR", "/tmp/charm-builds")
BUILT_CHARM = Path(CHARM_BUILD_DIR) / "juju-controller"


async def juju_status(unit, inner_model, filter=""):
    """Return juju staus in json format."""
    out = await model.async_run_on_unit(
        unit,
        "sudo -u ubuntu -H /snap/bin/juju status --format json -m {} {}".format(
            filter, inner_model
        ),
    )
    assert out["Code"] == "0", "Error: {}".format(out.get("Stderr"))
    data = json.loads(out["Stdout"])
    return data


async def check_for_inner_status(unit, inner_model, inner_app, target="active"):
    """Check if status of an app is equal to target."""
    status = await juju_status(unit, inner_model)
    apps = status.get("applications", {})
    inner = apps.get(inner_app, {})
    if (
        "application-status" in inner
        and inner["application-status"]["current"] == target
    ):
        return True
    return False


async def async_block_until_inner_app(unit, inner_model, app):
    """Wait for app to be active."""
    await model.async_block_until(
        lambda: check_for_inner_status(unit, inner_model, app),
        timeout=600,
        wait_period=2,
    )


block_until_inner_app = sync_wrapper(async_block_until_inner_app)


class JujuControllerTestBase(unittest.TestCase):
    """Base class for charm functional tests."""

    @classmethod
    def setUpClass(cls):
        """Set up test environment."""
        cls.model_name = model.get_juju_model()
        cls.test_config = lifecycle_utils.get_charm_config()
        model.block_until_all_units_idle()
        cls.unit = "ubuntu/0"
        cls.enable_ha()
        cls.deploy_juju_controller()
        cls.deploy_monitoring()

    @classmethod
    def enable_ha(cls):
        """Enable juju controller HA with 3 units."""
        out = model.run_on_unit(
            cls.unit, "sudo -u ubuntu -H /snap/bin/juju enable-ha -n 3"
        )
        assert out["Code"] == "0", "Error enabling ha: {}".format(out["Stderr"])

    @classmethod
    def run_remote_juju(cls, cmd, opts):
        """Run juju command on controller model."""
        cmd = "sudo -u ubuntu -H /snap/bin/juju {} -m controller {}".format(cmd, opts)
        out = model.run_on_unit(cls.unit, cmd)
        assert out["Code"] == "0", "Error running {}: {}".format(cmd, out["Stderr"])
        return out["Stdout"]

    @classmethod
    def deploy_juju_controller(cls):
        """Deploy juju-controller charm."""
        model.scp_to_unit(cls.unit, str(BUILT_CHARM), "/home/ubuntu", scp_opts="-r")
        cls.run_remote_juju(
            "deploy",
            "-n 3 --to 0,1,2 /home/ubuntu/juju-controller juju-controller",
        )
        block_until_inner_app(cls.unit, "controller", "juju-controller")

    @classmethod
    def deploy_monitoring(cls):
        """Deploy monitoring apps."""
        cls.run_remote_juju("deploy", "cs:nagios")
        cls.run_remote_juju("deploy", "cs:nrpe")
        cls.run_remote_juju(
            "add-relation",
            "juju-controller:nrpe-external-master nrpe:nrpe-external-master",
        )
        cls.run_remote_juju("add-relation", "nagios:monitors nrpe:monitors")
        block_until_inner_app(cls.unit, "controller", "nrpe")
        block_until_inner_app(cls.unit, "controller", "nagios")
        block_until_inner_app(cls.unit, "controller", "juju-controller")

    def remote_cat(self, machine, path):
        """Run cat command on a machine in the controller model."""
        cmd = (
            "sudo -u ubuntu -H /snap/bin/juju run -m controller --machine {} "
            "sudo cat {}"
        ).format(machine, path)
        out = model.run_on_unit(self.unit, cmd)
        assert out["Code"] == "0", "Error catting {} on {}: {}".format(
            path, machine, out["Stderr"]
        )
        return out["Stdout"]


class BasicJujucontrollerTest(JujuControllerTestBase):
    """Class for charm functional tests."""

    def test_nrpe_rel(self):
        """Check that nrpe check is rendered correctly."""
        expect = "/usr/local/lib/nagios/plugins/check_systemd.py juju-db"
        for u in (0, 1, 2):
            chk = self.remote_cat(u, "/etc/nagios/nrpe.d/check_juju-db.cfg")
            self.assertTrue(chk.find(expect) != -1)
