#!/usr/bin/python3
"""A collection of fixtures to aid with unit testing."""
import os

from charmhelpers.core import unitdata

import mock

import pytest

import responses

import yaml


# If layer options are used, add this to jujucontroller
# and import layer in lib_juju_controller
@pytest.fixture
def mock_layers(monkeypatch):
    """Mock layers."""
    import sys

    sys.modules["charms.layer"] = mock.Mock()
    sys.modules["reactive"] = mock.Mock()
    # Mock any functions in layers that need to be mocked here

    def options(layer):
        # mock options for layers here
        if layer == "example-layer":
            options = {"port": 9999}
            return options
        else:
            return None

    monkeypatch.setattr("lib_juju_controller.layer.options", options)


@pytest.fixture
def mock_hookenv_config(monkeypatch):
    """Mock the charm config."""
    import yaml

    def mock_config():
        cfg = {}
        charm_config_path = os.path.join(os.path.dirname(__file__), "../../config.yaml")
        yml = yaml.safe_load(open(charm_config_path))

        # Load all defaults
        for key, value in yml["options"].items():
            cfg[key] = value["default"]

        # Manually add cfg from other layers
        # cfg['my-other-layer'] = 'mock'
        return cfg

    monkeypatch.setattr("lib_juju_controller.hookenv.config", mock_config)


@pytest.fixture
def mock_remote_unit(monkeypatch):
    """Mock hookenv.remote_unit."""
    monkeypatch.setattr(
        "lib_juju_controller.hookenv.remote_unit", lambda: "unit-mock/0"
    )


@pytest.fixture
def mock_charm_dir(monkeypatch):
    """Provide a mocked hookenv.charm_dir."""
    monkeypatch.setattr(
        "lib_juju_controller.hookenv.charm_dir", lambda: "/mock/charm/dir"
    )


@pytest.fixture
def mock_unit_db(monkeypatch):
    """Mock the key value store."""
    mock_kv = mock.Mock()
    mock_kv.return_value = unitdata.Storage(path=":memory:")
    monkeypatch.setattr("lib_juju_controller.unitdata.kv", mock_kv)


@pytest.fixture
def agent_conf_sample_path():
    """Provide a sample agent.conf (path only)."""
    return os.path.join(os.path.dirname(__file__), "../samples/agent.conf")


@pytest.fixture
def agent_conf_sample(agent_conf_sample_path):
    """Provide a map representing the content of the sample agent.conf."""
    with open(agent_conf_sample_path, "r") as agentconffile:
        return yaml.safe_load(agentconffile)


@pytest.fixture
def jujucontroller(
    mock_hookenv_config,
    monkeypatch,
    agent_conf_sample,
):
    """Mock the JujuControllerHelper.

    Properties are initialized based on the contents of the agent.conf sample
    """
    from lib_juju_controller import JujuControllerHelper

    helper = JujuControllerHelper()

    monkeypatch.setattr(
        "lib_juju_controller.glob.glob", lambda _: ["/var/lib/juju/agents/machine-99"]
    )

    monkeypatch.setattr(
        "lib_juju_controller.JujuControllerHelper.agent_conf", agent_conf_sample
    )

    # Any other functions that load helper will get this version
    monkeypatch.setattr("lib_juju_controller.JujuControllerHelper", lambda: helper)

    return helper


@pytest.fixture
def mocked_responses():
    """Provide mock responses as a fixture.

    This allows tests to use the responses module for mocking interactions with
    the requests module.
    """
    with responses.RequestsMock() as response:
        yield response
