#!/usr/bin/python3
"""Pytest test cases for the check_jujucontroller.py NRPE check."""
import os
import sys

import pytest

import responses

PLUGINS_DIR = os.path.join(os.path.dirname(__file__), "../../files/plugins")
sys.path.append(PLUGINS_DIR)
import check_jujucontroller as nrpecheck  # noqa: E402,I100,I202


class TestCheckJujuController:
    """Class containing all tests for the controller NRPE check."""

    @pytest.mark.parametrize("version,expected", [("2.8.1", True), ("2.5.6", False)])
    def test_supported_function(self, version, expected):
        """Ensure the version check cuts off at 2.7.2."""
        assert nrpecheck._supported(version) is expected

    def test_no_pubagentconf(self, monkeypatch):
        """Return UNKNOWN if the required auxiliary files are not in place."""
        state, _ = nrpecheck.check_endpoint("/this/file/does/not/exist")
        assert state == nrpecheck.States.UNKNOWN

    def test_unsupported(self, monkeypatch, agent_conf_sample_path):
        """Checking an unsupported version should return UNKNOWN."""
        monkeypatch.setattr("check_jujucontroller._supported", lambda _: False)
        state, _ = nrpecheck.check_endpoint(agent_conf_sample_path)
        assert state == nrpecheck.States.UNKNOWN

    def test_connection_error(self, monkeypatch, agent_conf_sample_path):
        """Connection exceptions should be reported as CRITICAL."""
        monkeypatch.setattr(
            "check_jujucontroller.requests.get",
            lambda *args, **kwargs: exec('raise ConnectionError("foo")'),
        )
        state, msg = nrpecheck.check_endpoint(agent_conf_sample_path)
        assert state == nrpecheck.States.CRITICAL
        assert msg == "cannot connect to the health endpoint. Error: foo"

    def test_status_codes(
        self, jujucontroller, mocked_responses, agent_conf_sample_path
    ):
        """The endpoint should always return 200, regardless of controller state.

        Any other HTTP status should be considered an UNKNOWN state.
        This might change in the future.
        """
        mocked_responses.add(
            responses.GET,
            "https://localhost:{}/health".format(jujucontroller.apiport),
            body="foo",
            status=404,
        )
        state, _ = nrpecheck.check_endpoint(agent_conf_sample_path)
        assert state == nrpecheck.States.UNKNOWN

    @pytest.mark.parametrize(
        "response,expected_state",
        [(res, nrpecheck.States.OK) for res in nrpecheck.OKSTATUSES]
        + [(res, nrpecheck.States.WARNING) for res in nrpecheck.WARNSTATUSES]
        + [("fubar", nrpecheck.States.CRITICAL)],
    )
    def test_endpoint_response(
        self,
        response,
        expected_state,
        mocked_responses,
        agent_conf_sample_path,
        jujucontroller,
    ):
        """Check that all known reponse bodies cause the related NRPE status."""
        mocked_responses.add(
            responses.GET,
            "https://localhost:{}/health".format(jujucontroller.apiport),
            body=response,
            status=200,
        )
        state, _ = nrpecheck.check_endpoint(agent_conf_sample_path)
        assert state == expected_state
